﻿// Logger
// (c) 2024 Stephane

// This is an include file script for Directory Opus.
// See https://www.gpsoft.com.au/DScripts/redirect.asp?page=scripts for development information.



// Called by Directory Opus to initialize the include file script
function OnInitIncludeFile(initData)
{
	initData.name = "Logger";
	initData.version = "0.8.2";
	initData.copyright = "(c) 2024 Stephane";
//	initData.url = "https://resource.dopus.com/c/buttons-scripts/16";
	initData.desc = "Console & File logger";
	initData.min_version = "13.9.2";
	initData.shared = true;
}



var LoggerNS = (function() {
	// Emulates namespace (ns = LoggerNS)
	var my = {};

	// =============================================================
	// Enums
	// =============================================================
	
	// LogLevel enum
	// ---
	var LogLevel;
	(function (LogLevel) {
	    LogLevel[LogLevel["Trace"]	= 0] = "Trace";		// TRC
	    LogLevel[LogLevel["Debug"]	= 1] = "Debug";		// DBG
	    LogLevel[LogLevel["Info"]	= 2] = "Info";		// NFO
	    LogLevel[LogLevel["Warn"]	= 3] = "Warn";		// WRN
		LogLevel[LogLevel["Error"]	= 4] = "Error";		// ERR
		LogLevel[LogLevel["Fatal"]	= 5] = "Fatal";		// FAT
	})(LogLevel || (LogLevel = {}));
	// Exposing LogLevel outside of namespace
	my.LogLevel = LogLevel;
	// ---

	// TargetType enum
	var TargetType;
	(function (TargetType) {
	    TargetType[TargetType["Console"]	= 0] = "Console";
	    TargetType[TargetType["File"]		= 1] = "File";
	})(TargetType || (TargetType = {}));
	// Exposing TargetType outside of namespace
	my.TargetType = TargetType;

	// =============================================================
	// Global settings
	// =============================================================
	my.debug = false;


	// =============================================================
	// Classes
	// =============================================================
	
	// =====
	// LoggerTarget class
	// =============================================================
	my.LoggerTarget = function /* class */ LoggerTarget(name, minLevel, maxLevel, type, path, filename, archiveDelay, deleteDelay) {
		// Internal info output function
		this.info = function(msg) { if (my.debug) DOpus.Output("<i><font color=#004fb1 bgcol=#eec057>LoggerTarget: " + msg); };

		// Properties that can be set through constructor
		// ---
		this.name = name || "dummy";
		this.minLevel = minLevel || my.LogLevel.Trace;
		this.maxLevel = maxLevel || my.LogLevel.Fatal;
		this.targetType = type || my.TargetType.Console;
		this.logPath = "";
		if (path !== undefined) {
			this.info("Path provided : " + path);
			this.logPath = path;
		}
		else if (type == TargetType.File) {
			// DOpus.Output("path not provided");
			if (DOpus.FSUtil.Exists("/cLoggerPath/")) this.logPath = "/cLoggerPath/";
			else this.logPath = "/temp/";
		}
		// DOpus.Output("logPath ='" + this.logPath + "' - Exists ? =>" + DOpus.FSUtil.Exists(this.logPath));
		this.logFilename = filename || "";
		// if (filename !== undefined) {
		// 	this.info("Filename provided : " + filename);
		// 	this.logFilename = filename;
		// }

		this.fullLogfilePath = "";	// will be set by logger on first log for TargetType.File
		this.disabled = false;		// internal - set to true when logger can not create the log file
		
		this.archiveDelay = archiveDelay || 30;
		this.deleteDelay = deleteDelay || 365;
		if (this.archiveDelay > this.deleteDelay) this.archiveDelay = this.deleteDelay
		
		// Other properties (need to be modified directly once the object is constructed, if required)
		// ---
		this.version = "0.1";
		this.defaultTarget = false;
		this.archiveOlder = true;
		this.layout = {
			Separator: "|",
			UseColor: (this.targetType == TargetType.Console) ? true : false,
			IncludeInstanceId: true,
			IncludeDateTime: true,
			IncludeCallsite: true,
			IncludeLogLevel: true
		}

		// Default color map	
		this.colors = DOpus.Create.Map();
		// map for Trace		
		this.colors(LogLevel.Trace) = DOpus.Create.Map();
		this.colors(LogLevel.Trace)("date")		= { fg: "none", bg: "#2a2a2a" };
		this.colors(LogLevel.Trace)("id")		= { fg: "#86abdd", bg: "none" };
		this.colors(LogLevel.Trace)("caller")	= { fg: "#0061d8", bg: "#d4d4d4" };
		this.colors(LogLevel.Trace)("level")	= { fg: "#a0a0a0", bg: "none" };
		this.colors(LogLevel.Trace)("msg")		= { fg: "#8a8a8a", bg: "none" };

		// map for Debug		
		this.colors(LogLevel.Debug) = DOpus.Create.Map();
		this.colors(LogLevel.Debug)("date")		= { fg: "none", bg: "#2a2a2a" };
		this.colors(LogLevel.Debug)("id")		= { fg: "#86abdd", bg: "none" };
		this.colors(LogLevel.Debug)("caller")	= { fg: "#0061d8", bg: "#d4d4d4" };
		this.colors(LogLevel.Debug)("level")	= { fg: "#eaeaea", bg: "#5c5c5c" };
		this.colors(LogLevel.Debug)("msg")		= { fg: "#cbcbcb", bg: "none" };

		// map for Info		
		this.colors(LogLevel.Info) = DOpus.Create.Map();
		this.colors(LogLevel.Info)("date")		= { fg: "none", bg: "#2a2a2a" };
		this.colors(LogLevel.Info)("id")		= { fg: "#86abdd", bg: "none" };
		this.colors(LogLevel.Info)("caller")	= { fg: "#0061d8", bg: "#d4d4d4" };
		this.colors(LogLevel.Info)("level")		= { fg: "#000000", bg: "#6f9cf2" };
		this.colors(LogLevel.Info)("msg")		= { fg: "#16aae9", bg: "none" };

		// map for Warn		
		this.colors(LogLevel.Warn) = DOpus.Create.Map();
		this.colors(LogLevel.Warn)("date")		= { fg: "none", bg: "#2a2a2a" };
		this.colors(LogLevel.Warn)("id")		= { fg: "#86abdd", bg: "none" };
		this.colors(LogLevel.Warn)("caller")	= { fg: "#0061d8", bg: "#d4d4d4" };
		this.colors(LogLevel.Warn)("level")		= { fg: "#ff8000", bg: "none" };
		this.colors(LogLevel.Warn)("msg")		= { fg: "#ff9800", bg: "none" };
		
		// map for Error
		this.colors(LogLevel.Error) = DOpus.Create.Map();
		this.colors(LogLevel.Error)("date")		= { fg: "none", bg: "#2a2a2a" };
		this.colors(LogLevel.Error)("id")		= { fg: "#86abdd", bg: "none" };
		this.colors(LogLevel.Error)("caller")	= { fg: "#0061d8", bg: "#d4d4d4" };
		this.colors(LogLevel.Error)("level")	= { fg: "#ff0000", bg: "none" };
		this.colors(LogLevel.Error)("msg")		= { fg: "#ff0000", bg: "none" };
		
		// map for Fatal
		this.colors(LogLevel.Fatal) = DOpus.Create.Map();
		this.colors(LogLevel.Fatal)("date")		= { fg: "none", bg: "#2a2a2a" };
		this.colors(LogLevel.Fatal)("id")		= { fg: "#86abdd", bg: "none" };
		this.colors(LogLevel.Fatal)("caller")	= { fg: "#0061d8", bg: "#d4d4d4" };
		this.colors(LogLevel.Fatal)("level")	= { fg: "#f8f8f8", bg: "#ff0000" };
		this.colors(LogLevel.Fatal)("msg")		= { fg: "#f8f8f8", bg: "#ff0000" };


		// Methods
		// ---

		// ExportToJSON : stringifies (serializes) the object
		this.ExportToJSON = function() {
			var res = JSON.stringify(this, null, 2);		// Handles all basic objects (strings, integers/enums, ... including the layout JSON object
			//this.info("First stringify : " + res);
			var obj = JSON.parse(res);

			// Since it does not handle (at all) the DOpus.Map, we need to manage this
			// We will transform this map into a JSON object
			var colors = {};	
			var allTypes = [ "date", "id", "caller", "level", "msg" ];
			for (var level = LogLevel.Trace; level <= LogLevel.Fatal; level++) {
				colors[level] = {};
				for (var i = 0; i < allTypes.length; i++) {
					colors[level][allTypes[i]] = { fg: this.colors(level)(allTypes[i]).fg, bg: this.colors(level)(allTypes[i]).bg };
				}
			}

			obj.colors = colors;
			res = JSON.stringify(obj, null, 2);
			//res = JSON.stringify(obj);
			//this.info("Final export to JSON: " + res);
			return res;
		}

		// ImportFromJSON : parses JSON to set the properties of the object (deserializes)
		this.ImportFromJSON = function(json) {
			// this.info("Importing JSON");
			//this.info("Importing JSON\n" + json);
			var obj = JSON.parse(json);
			try {	// could fail if we try to convert from an old json that didn't have all the same properties as the object now has
				this.name 			= obj.name;
				this.minLevel		= obj.minLevel;
				this.maxLevel		= obj.maxLevel;
				this.targetType		= obj.targetType;
				this.logPath		= obj.logPath;
				this.logFilename	= obj.logFilename;
				this.archiveDelay	= + obj.archiveDelay;
				this.deleteDelay	= + obj.deleteDelay;
				//this.version		= json.version;	// is this true ??? should we leave the version fiels as is ???
				this.defaultTarget	= obj.defaultTarget;
				this.archiveOlder	= obj.archiveOlder;
				this.layout			= obj.layout;

				var s = obj.colors;
				this.colors = DOpus.Create.Map();
				var allTypes = [ "date", "id", "caller", "level", "msg" ];
				for (var level = LogLevel.Trace; level <= LogLevel.Fatal; level++) {
					this.colors(level) = DOpus.Create.Map();
					for (var i = 0; i < allTypes.length; i++) {
						//this.info("Adding color for level=" + level + " / type=" + allTypes[i] + " : " + obj.colors[level][allTypes[i]].fg + "/" + obj.colors[level][allTypes[i]].bg);
						this.colors(level)(allTypes[i]) = { fg: obj.colors[level][allTypes[i]].fg, bg: obj.colors[level][allTypes[i]].bg };
					}
				}
				this.info("Import done - name = " + this.name);
			}
			catch(e) {
				this.info("Exception catched : " + e);
			}
		};

		// clone function
		this.Clone = function() {
			return my.BuildTargetFromJSON(this.ExportToJSON());
		};

		// ColorItem : Modifies the message to add "coloration" attributes
		this.ColorItem = function(loglevel, type, msg) {
			if (this.layout.UseColor == false || this.targetType != TargetType.Console)
				return msg;

			// else

			//DOpus.Output("dbg0:" + loglevel + ":" + type + ":" + msg);
			if (!this.colors.exists(loglevel)) return "</#>" + msg;
			//DOpus.Output("dbg1");
			
			var color = this.colors(loglevel);
			//DOpus.Output("dbg1b : " + color.count);
			if (!color.exists(type)) return "</#>" + msg;
			//DOpus.Output("dbg2");
			
			var typeColor = color(type);
			if (typeColor.fg == "none" && typeColor.bg == "none") return "</#>" + msg;
			//DOpus.Output("dbg3");
			return "<font " + ((typeColor.fg != "none")?"color="+typeColor.fg+" ":"") + ((typeColor.bg != "none")?"bgcol="+typeColor.bg:"") + ">" + msg + "</#>";
		};



		// toString
		this.toString = function() { return this.name; };
	};


	// =====
	// LoggerConfiguration class ==> useles ???
	// =============================================================

	my.LoggerConfiguration = function /* class */ LoggerConfiguration() {
		// Internal info output function
		this.info = function(msg) { DOpus.Output("<i><font color=#004fb1 bgcol=#eec057>LoggerConfiguration: " + msg); };

		// Properties & methods
		// ---
		this.targets = DOpus.Create.Vector();

		
		this.AddTarget = function(target) { this.targets.push_back(target); };

		// Gets first target matching provided name
		this.getFirst = function(nameToMatch) {
			for (var e = new Enumerator(this.targets); !e.atEnd(); e.moveNext()) {
				if (e.item().name == nameToMatch) return e.item();
			}
			return undefined;
		};

		// Gets first index of target matching provided name
		this.getFirstIndex = function(nameToMatch) {
			for (var i = 0; i < this.targets.length; i++) {
				if (this.targets[i].name == nameToMatch) return i;
			}
			return -1;
		};


		// Save to DOpus global vars (CommonLoggerConfig) and persist to disk
		this.SavePersist = function() {
			var targetsToSave = DOpus.Create.Vector();
			for (var e = new Enumerator(this.targets); !e.atEnd(); e.moveNext()) {
				targetsToSave.push_back(e.item().ExportToJSON());
			}
			DOpus.Vars.Set("CommonLoggerConfig", targetsToSave);
			DOpus.Vars("CommonLoggerConfig").persist = true;
		};

		// Load from DOpus global vars (CommonLoggerConfig). Returns undefined when variable not found
		this.Load = function() {
			if (!DOpus.Vars.Exists("CommonLoggerConfig")) {
				this.info("Unable to retrieve CommonLoggerConfig from vars. Creating default targets.");
				var consoleTarget = new my.LoggerTarget("Default Console", LoggerNS.LogLevel.Trace, LoggerNS.LogLevel.Fatal, LoggerNS.TargetType.Console);
				consoleTarget.defaultTarget = true;
				consoleTarget.layout.IncludeDateTime = false;
				var fileTarget = new my.LoggerTarget("Default File", LoggerNS.LogLevel.Trace, LoggerNS.LogLevel.Fatal, LoggerNS.TargetType.File, "/cLoggerPath/");
				fileTarget.defaultTarget = true;
				this.targets.clear();
				this.AddTarget(consoleTarget);
				this.AddTarget(fileTarget);
				this.SavePersist();

				return this;
			}
			else {
				this.targets.clear();
				var savedVector = DOpus.Vars.Get("CommonLoggerConfig");
				for (var e = new Enumerator(savedVector); !e.atEnd(); e.moveNext()) {
					// DOpus.Output("Item: " + e.item());
					var target = my.BuildTargetFromJSON(e.item());
					this.AddTarget(target);

					// DOpus.Output("item: " + JSON.stringify(target));
					// DOpus.Output("LoggerTarget name: " + target.name);
					// DOpus.Output("Typeof LoggerTarget: " + (typeof target));
					// DOpus.Output("DOpus.Typeof LoggerTarget: " + (DOpus.TypeOf(target)));
					// DOpus.Output("value stringify: " + JSON.stringify(target));
					// DOpus.Output("Instanceof LoggerTarget: " + (target instanceof LoggerNS.LoggerTarget));
					// DOpus.Output("name : " + target.name);				
				}
				return this;
			}
		};


		// ExportToJSON : stringifies (serializes) the object
		this.ExportToJSON = function() {
			this.info("exportToJSON");
			var res = JSON.stringify(this);		// Handles all basic objects (strings, integers/enums, ... including the layout JSON object
			//this.info("First stringify : " + res);
			var obj = JSON.parse(res);
			obj.nbTargets = this.targets.length;

/*			var targets = {};
			for (var i = 0; i < this.targets.length; i++) {
				targets[i] = {};
				targets[i].json =JSON.parse(this.targets[i].ExportToJSON());
			}
*/
			var targets = [];
			for (var i = 0; i < this.targets.length; i++) {
				targets.push(JSON.parse(this.targets[i].ExportToJSON()));
			}

			obj.targets = targets;
			res = JSON.stringify(obj, null, 2);
			//this.info("Final export to JSON: " + res);
			return res;
		}

		// ImportFromJSON : parses JSON to set the properties of the object (deserializes)
		this.ImportFromJSON = function(json) {
			this.info("Importing JSON");
			//this.info("Importing JSON\n" + json);
			var obj = JSON.parse(json);
			
			try {	// could fail if we try to convert from an old json that didn't have all the same properties as the object now has
				var nbTargets = obj.nbTargets;
				var t = obj.targets;
/*				var targets = DOpus.Create.Vector();
				for (var i = 0; i < nbTargets; i++) {
					//this.info("Adding color for level=" + level + " / type=" + allTypes[i] + " : " + obj.colors[level][allTypes[i]].fg + "/" + obj.colors[level][allTypes[i]].bg);
					targets.push_back(my.BuildTargetFromJSON(JSON.stringify(obj.targets[i].json)));
				}
*/				var targets = DOpus.Create.Vector();
				for (var i = 0; i < obj.targets.length; i++) {
					//this.info("Adding color for level=" + level + " / type=" + allTypes[i] + " : " + obj.colors[level][allTypes[i]].fg + "/" + obj.colors[level][allTypes[i]].bg);
					targets.push_back(my.BuildTargetFromJSON(JSON.stringify(obj.targets[i])));
				}
				

				this.targets = targets;
				this.info("Import done - nbTargets = " + this.targets.length);
			}
			catch(e) {
				this.info("Exception catched : " + e);
			}
		};

	};

	// =====
	// Logger class
	// =============================================================
	my.Logger = function /* class */ Logger() {
		// Internal info output function
		this.infoMessage = function(msg, force) { if (my.debug||force) DOpus.Output("<i><font color=#004fb1 bgcol=#eec057>Logger: " + msg); };

		// Get targets from arguments
		this.targets = DOpus.Create.Vector();
		this.infoMessage("args length = " + arguments.length + " / type of first = " + typeof arguments[0]);
		// see https://stackoverflow.com/questions/1486300/javascript-type-of-custom-object
		if (arguments.length == 1 && arguments[0] instanceof Array) {
			this.infoMessage("Logger constructor > Array argument : looping through array content");
			// Loop through array to get instances (see https://stackoverflow.com/questions/4116608/pass-unknown-number-of-arguments-into-javascript-function)
			var args = Array.prototype.slice.call(arguments);
			for (var i=0; i<args.length; i++)
				this.targets.push_back(args[i]);
		}
		else if (arguments.length == 1 && arguments[0] instanceof my.LoggerConfiguration) {
			this.infoMessage("Logger Constructor > LoggerConfiguration as templates");
			var cfg = arguments[0];
			for (var i = 0; i < cfg.targets.length; i++) {
				this.targets.push_back(cfg.targets[i]);
				this.infoMessage("Pushed new copied target '" + cfg.targets[i].name + "' / logFilename = " + cfg.targets[i].logFilename);
			}
		}
		else {
			this.infoMessage("Logger constructor > List of targets provided : looping through arguments (#args = " + arguments.length + ")");
			for (var i=0; i<arguments.length; i++) 
				this.targets.push_back(arguments[i]);
		}

		// Check for "all targets are disabled ==> create & add a console standard target"


		// log function aliases
		this.trace	= function(msg) { this.log(LogLevel.Trace, msg, this.GetCaller()); };
		this.debug	= function(msg) { this.log(LogLevel.Debug, msg, this.GetCaller()); };
		this.info	= function(msg) { this.log(LogLevel.Info, msg, this.GetCaller()); };
		this.warn	= function(msg) { this.log(LogLevel.Warn, msg, this.GetCaller()); };
		this.error	= function(msg) { this.log(LogLevel.Error, msg, this.GetCaller()); };
		this.fatal	= function(msg) { this.log(LogLevel.Fatal, msg, this.GetCaller()); };

		this.log = function(level, msg, caller) {
			// Define everything that will be identical for all targets (caller, timestamp, etc ...)
			var timestamp = this.LoggerGetDateHour();
			if (caller == undefined) caller = this.GetCaller();

			//TODO : Loop through all targets : Console goes to DOpus.Output // File goes to ... file :)
			// DOpus.Output("Iteration through targets (#" + this.targets.length + ")");
			var fsu = DOpus.FSUtil;
			var cmd = DOpus.Create.Command();
			var oneLogDone = false;
			for (var i = 0; i < this.targets.length; i++) {
				var target = this.targets[i];
				if (target.targetType == TargetType.Console && !target.disabled) {
					var out = this.formatMessage(target, caller, level, timestamp, msg);
					DOpus.Output(out, false, false);
					oneLogDone = true;
				}
				else if (!target.disabled) { // type file
					if (target.fullLogfilePath == "") {
						// Create Path to LogFile if it does not exists
						cmd.RunCommand("CreateFolder Name=\"" + target.logPath + "\"");
						if (!DOpus.FSUtil.Exists(target.logPath)) {
							this.infoMessage("Warning : Unable to find log folder ... disabling target.", true);
							target.disabled = true;
							continue;
						}
						target.fullLogfilePath = target.logPath + "\\" + target.logFilename + " - " + this.formatFileDate(timestamp) + " - " + this.Id + ".log";


						if (cmd.CommandList("s").exists("ManageOldLogFiles"))
							cmd.RunCommandAsync('ManageOldLogFiles SCRIPT_PATH="' + target.logPath + '" SCRIPT_NAME="' + target.logFilename + '" ARCHIVE_DELAY=' + target.archiveDelay + ' DELETE_DELAY=' + target.deleteDelay);
						else
							this.infoMessage("ManageOldLogFiles command not installed - Unable to try and cleanup log files. Install 'CommonLogger.js' in order to solve.", true);
					}

					var file = fsu.OpenFile(target.fullLogfilePath, "rwXo");
					if (file.error != 0) {
						this.infoMessage("Unable to open file '" + target.logFilename + "' for logging.");
						file.Close();
						continue;
					}

					// DOpus.Output(" >> '" + oFile + "' created : " + typeof oFile + " | Dopus type = " + DOpus.TypeOf(oFile));
					var myText = this.formatMessage(target, caller, level, timestamp, msg);
					if (myText.length > 0) {
						file.Seek(0, "e");
						var s = file.Write("\n" + myText);
						file.Close();
						oneLogDone = true;
						// DOpus.Output("oFile write size = " + s);
						// target.file.Close();		// We don't close it to avoid reopening for each log.
					}
					// else DOpus.Output("Empty message - skipping");
				}
			}
			if (!oneLogDone) {
				var fallbackTarget = new my.LoggerTarget("Fallback Target");
				var out = this.formatMessage(fallbackTarget, caller, level, timestamp, msg);
				DOpus.Output(out, false, false);

				// dout("All targets disabled");
			}
		};

		this.formatMessage = function(loggerTarget, caller, level, timestamp, msg) {
			if (loggerTarget.minLevel > level || loggerTarget.maxLevel < level) {
				// DOpus.Output("not in range"); 
				return "";
			}
			// else:
			var outArray = new Array();
			// DOpus.Output("Processing target : " + loggerTarget.name);
			var sep = loggerTarget.layout.Separator;

			if (loggerTarget.layout.IncludeDateTime) outArray.push((loggerTarget.TargetType == my.TargetType.Console)?"</#>":"" + loggerTarget.ColorItem(level, "date", this.formatDateHour(timestamp)));
			if (loggerTarget.layout.IncludeInstanceId) outArray.push(loggerTarget.ColorItem(level, "id", this.Id));
			if (loggerTarget.layout.IncludeCallsite) outArray.push(loggerTarget.ColorItem(level, "caller", caller));
			if (loggerTarget.layout.IncludeLogLevel) outArray.push(loggerTarget.ColorItem(level, "level", LogLevel[level]));

			if (msg.match(/<$/)) msg += " ";	// # fix issue when last character is a '<'
			outArray.push("" + loggerTarget.ColorItem(level, "msg", msg + " "));
			return outArray.join(sep);		
		};

		// Inner tool functions & properties
		this.maxId = 10000;
		this.Id = Math.floor(Math.random()*this.maxId).toString().padStart(4,"0");


		this.GetCaller = function() {
			try { return arguments.callee.caller.caller.toString().match(/function\s+([^\s\(]+)/)[1]; }
			catch (e) { return "<<Unknown caller function>>"; } 
		};

		this.LoggerGetDateHour = function() {
			var d = new Date();
			var vYear = d.getFullYear();
			var vMonth = "" + (d.getMonth() + 1);
			var vDay = "" + d.getDate();
			var vHour = "" + d.getHours();
			var vMinute = "" + d.getMinutes();
			var vSeconds = "" + d.getSeconds();
			var vMsecs = "" + d.getMilliseconds();
		
			return {	year: vYear,
						month: vMonth.padStart(2, '0'),
						day: vDay.padStart(2, '0'),
						hour: vHour.padStart(2, '0'),
						minute: vMinute.padStart(2, '0'),
						second: vSeconds.padStart(2, '0'),
						msecond: vMsecs.padStart(3, '0')
					};	
		};
		
		this.formatDateHour = function(oDH) {
			return oDH.year + "." + oDH.month + "." + oDH.day + "-" + oDH.hour + ":" + oDH.minute + ":" + oDH.second + "." + oDH.msecond;
		};

		this.formatFileDate = function(oDH) {
			return oDH.year + "-" + oDH.month + "-" + oDH.day;
		};
	};


	// =====
	// Helper/static functions
	// =============================================================

	// ===
	// Function to build target from template
	// ===
	my.BuildTargetFromJSON = function(json) {
		// Internal info output function
		this.info = function(msg) { if (my.debug) DOpus.Output("<i><font color=#004fb1 bgcol=#eec057>BuildTargetFromJSON: " + msg); };
		// this.info("Building from : " + json);
		var target = new my.LoggerTarget();
		target.ImportFromJSON(json);
		return target;
	};


	// ===
	// Function to build target from template
	// ===
	my.BuildTargetFromTemplate = function(templateName, filename) {
		// Internal info output function
		this.info = function(msg, force) { if (my.debug||force) DOpus.Output("<i><font color=#004fb1 bgcol=#eec057>BuildTargetFromTemplate: " + msg); };

		// TODO
		// Get Global var containing all targets
		
		var allTemplates = new my.LoggerConfiguration();
		allTemplates.Load();
		var index = allTemplates.getFirstIndex(templateName);
		if (index == -1) {
			this.info("FATAL ERROR: No template found with name '" + templateName + "'. RUN the ConfigLogger or fix template name.", true);
			var target =  new my.LoggerTarget();
			target.disabled = true;
			return target;
		}
		else {
			var target = allTemplates.targets[index].Clone();
			target.logFilename = filename;
			return target;
		} 
	};

	// ===
	// Function to build a configuration from all "Default" targets, and applying the log filename to each "File" target. Output is ready to be given as an input to a Logger.
	// ===
	my.BuildDefaultConfigurations = function(targetLogFilename) {
		var allTemplates = new my.LoggerConfiguration();
		var returnConfig = new my.LoggerConfiguration();
		allTemplates.Load();
		// dout("templates loaded"); dout("target = " + targetLogFilename);
		var sanitizedFilename = sanitizeFilename(targetLogFilename, "_");
		for (var i = 0; i < allTemplates.targets.length; i++) {
			if (!allTemplates.targets[i].defaultTarget) continue;
			// else
			var newTarget = allTemplates.targets[i].Clone();
			// DOpus.Output("Cloning ... for type = " + newTarget.targetType);
			if (newTarget.targetType == TargetType.File) {
				// DOpus.Output(" >> Setting filename");
				newTarget.logFilename = sanitizedFilename;
			}
			returnConfig.AddTarget(newTarget);
		}
		return returnConfig;
	};

	// ===
	// Function to create a Logger built upon all default configurations, applying the log filename to each "File" target
	// ===
	my.BuildDefaultLogger = function(targetLogFilename) {
		var cfg = my.BuildDefaultConfigurations(targetLogFilename);
		return new my.Logger(cfg);
	};


	// ===
	// Function to get LogLevel value from its string name
	// ===
	my.GetLogLevelValue = function(strLevel) {
		for (var i = LogLevel.Trace; i <= LogLevel.Fatal; i++) {
			if (LogLevel[i] == strLevel) return i;
		}
		return -1;
	};


	// End of Namespace
	return my;
}());

/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////
// End Of namespace LoggerNS
/////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////
var print_names = function() {
	if (arguments.length == 0) DOpus.Output("No arguments provided");
	else if (arguments.length == 1) {
		DOpus.Output("Type : " + typeof arguments[0]);
	}
	else {
		DOpus.Output(typeof arguments);
		for (var i=0; i<arguments.length; i++) DOpus.Output(arguments[i]);
		var args = Array.prototype.slice.call(arguments);
		DOpus.Output(arguments.length);
		DOpus.Output(arguments[0]);
	}
}

///////////////////////////////////////////////////////////////////////////
if (!String.prototype.padStart) {
    String.prototype.padStart = function padStart(targetLength,padString) {
        targetLength = targetLength>>0; //truncate if number or convert non-number to 0;
        padString = String((typeof padString !== 'undefined' ? padString : ' '));
        if (this.length > targetLength) {
            return String(this);
        }
        else {
            targetLength = targetLength-this.length;
            if (targetLength > padString.length) {
                padString += padString.repeat(targetLength/padString.length); //append to original to ensure we are longer than needed
            }
            return padString.slice(0,targetLength) + String(this);
        }
    };
}

///////////////////////////////////////////////////////////////////////////
if (!String.prototype.repeat) {
  String.prototype.repeat = function(count) {
    'use strict';
    if (this == null) {
      throw new TypeError('can\'t convert ' + this + ' to object');
    }
    var str = '' + this;
    count = +count;
    if (count != count) {
      count = 0;
    }
    if (count < 0) {
      throw new RangeError('repeat count must be non-negative');
    }
    if (count == Infinity) {
      throw new RangeError('repeat count must be less than infinity');
    }
    count = Math.floor(count);
    if (str.length == 0 || count == 0) {
      return '';
    }
    // Ensuring count is a 31-bit integer allows us to heavily optimize the
    // main part. But anyway, most current (August 2014) browsers can't handle
    // strings 1 << 28 chars or longer, so:
    if (str.length * count >= 1 << 28) {
      throw new RangeError('repeat count must not overflow maximum string size');
    }
    var maxCount = str.length * count;
    count = Math.floor(Math.log(count) / Math.log(2));
    while (count) {
       str += str;
       count--;
    }
    str += str.substring(0, maxCount - str.length);
    return str;
  }
}

///////////////////////////////////////////////////////////////////////////
function sanitizeFilename(input, replacement) {
	// inspired from node sanitize filename : https://github.com/parshap/node-sanitize-filename/blob/master/index.js
	if (typeof input !== 'string') {
		throw new Error('Input must be string');
	}

	var illegalRe = /[\/\?<>\\:\*\|"]/g;
	var controlRe = /[\x00-\x1f\x80-\x9f]/g;
	var reservedRe = /^\.+$/;
	var windowsTrailingRe = /[\. ]+$/;

	var sanitized = input
		.replace(illegalRe, replacement)
		.replace(controlRe, replacement)
		.replace(reservedRe, replacement)
		.replace(windowsTrailingRe, replacement);
	return sanitized;
}